/*
 * Copyright (C) 2008 Andre Noll <maan@tuebingen.mpg.de>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file afh.c Paraslash's standalone audio format handler tool. */

#include <regex.h>

#include "para.h"
#include "string.h"
#include "afh.cmdline.h"
#include "fd.h"
#include "afh.h"
#include "error.h"
#include "version.h"
#include "ggo.h"

static struct afh_args_info conf;
INIT_AFH_ERRLISTS;

static int loglevel;
INIT_STDERR_LOGGING(loglevel)

static void print_info(int audio_format_num, struct afh_info *afhi)
{
	char *msg;

	afh_get_afhi_txt(audio_format_num, afhi, &msg);
	printf("%s", msg);
	free(msg);
}

static void print_chunk_table(struct afh_info *afhi)
{
	int i;

	if (conf.parser_friendly_given) {
		printf("chunk_table: ");
		for (i = 0; i <= afhi->chunks_total; i++)
			printf("%u ", afhi->chunk_table[i]);
		printf("\n");
		return;
	}
	for (i = 1; i <= afhi->chunks_total; i++) {
		struct timeval tv;
		long unsigned from, to;
		tv_scale(i - 1, &afhi->chunk_tv, &tv);
		from = tv2ms(&tv);
		tv_scale(i, &afhi->chunk_tv, &tv);
		to = tv2ms(&tv);
		printf("%d [%lu.%03lu - %lu.%03lu] %u - %u (%u)\n", i - 1,
			from / 1000, from % 1000, to / 1000, to % 1000,
			afhi->chunk_table[i - 1], afhi->chunk_table[i],
			afhi->chunk_table[i] - afhi->chunk_table[i - 1]);
	}
}

__noreturn static void print_help_and_die(void)
{
	struct ggo_help h = DEFINE_GGO_HELP(afh);
	int d = conf.detailed_help_given;
	unsigned flags = d? GPH_STANDARD_FLAGS_DETAILED : GPH_STANDARD_FLAGS;

	ggo_print_help(&h, flags);
	printf("supported audio formats: %s\n", AUDIO_FORMAT_HANDLERS);
	exit(EXIT_SUCCESS);
}

/**
 * The main function of para_afh.
 *
 * \param argc Usual argument count.
 * \param argv Usual argument vector.
 *
 * \return \p EXIT_FAILURE or \p EXIT_SUCCESS.
 */
int main(int argc, char **argv)
{
	int i, ret, audio_format_num, fd;
	void *audio_file_data;
	size_t audio_file_size;
	struct afh_info afhi;

	afh_cmdline_parser(argc, argv, &conf);
	loglevel = get_loglevel_by_name(conf.loglevel_arg);
	version_handle_flag("afh", conf.version_given);
	if (conf.help_given || conf.detailed_help_given)
		print_help_and_die();
	afh_init();
	ret = -E_AFH_SYNTAX;
	if (conf.inputs_num == 0)
		goto out;
	for (i = 0; i < conf.inputs_num; i++) {
		int ret2;
		ret = mmap_full_file(conf.inputs[i], O_RDONLY, &audio_file_data,
			&audio_file_size, &fd);
		if (ret < 0) {
			PARA_ERROR_LOG("failed to mmap \"%s\"\n", conf.inputs[i]);
			goto out;
		}
		ret = compute_afhi(conf.inputs[i], audio_file_data, audio_file_size,
			fd, &afhi);
		if (ret >= 0) {
			audio_format_num = ret;
			printf("File %d: %s\n", i + 1, conf.inputs[i]);
			print_info(audio_format_num, &afhi);
			if (conf.chunk_table_given)
				print_chunk_table(&afhi);
			printf("\n");
			clear_afhi(&afhi);
		}
		close(fd);
		ret2 = para_munmap(audio_file_data, audio_file_size);
		if (ret2 < 0 && ret >= 0)
			ret = ret2;
		if (ret < 0)
			break;
	}
out:
	if (ret < 0)
		PARA_ERROR_LOG("%s\n", para_strerror(-ret));
	return ret < 0? EXIT_FAILURE : EXIT_SUCCESS;
}
