/*
 * Copyright (C) 2005 Andre Noll <maan@tuebingen.mpg.de>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

#include "para.h"
/** \file time.c Helper functions for dealing with time values. */

/**
 * Convert struct timeval to milliseconds.
 *
 * \param tv The time value value to convert.
 *
 * \return The number of milliseconds in \a tv.
 */
long unsigned tv2ms(const struct timeval *tv)
{
	return tv->tv_sec * 1000 + (tv->tv_usec + 500)/ 1000;
}

/**
 * Convert milliseconds to a struct timeval.
 *
 * \param n The number of milliseconds.
 * \param tv Result pointer.
 */
void ms2tv(long unsigned n, struct timeval *tv)
{
	tv->tv_sec = n / 1000;
	tv->tv_usec = (n % 1000) * 1000;
}

/**
 * Compute the difference of two time values.
 *
 * \param b Minuend.
 * \param a Subtrahend.
 * \param diff Result pointer.
 *
 * If \a diff is not \p NULL, it contains the absolute value |\a b - \a a| on
 * return.
 *
 * \return If \a b < \a a, this function returns -1, otherwise it returns 1.
 */
int tv_diff(const struct timeval *b, const struct timeval *a,
		struct timeval *diff)
{
	int ret = 1;

	if ((b->tv_sec < a->tv_sec) ||
		((b->tv_sec == a->tv_sec) && (b->tv_usec < a->tv_usec))) {
		const struct timeval *tmp = a;
		a = b;
		b = tmp;
		ret = -1;
	}
	if (!diff)
		return ret;
	diff->tv_sec = b->tv_sec - a->tv_sec;
	if (b->tv_usec < a->tv_usec) {
		diff->tv_sec--;
		diff->tv_usec = 1000 * 1000 - a->tv_usec + b->tv_usec;
	} else
		diff->tv_usec = b->tv_usec - a->tv_usec;
	return ret;
}

/**
 * Add two time values.
 *
 * \param a First addend.
 * \param b Second addend.
 * \param sum Contains the sum \a a + \a b on return.
 */
void tv_add(const struct timeval *a, const struct timeval *b,
		struct timeval *sum)
{
	sum->tv_sec = a->tv_sec + b->tv_sec;
	if (a->tv_usec + b->tv_usec >= 1000 * 1000) {
		sum->tv_sec++;
		sum->tv_usec = a->tv_usec + b->tv_usec - 1000 * 1000;
	} else
		sum->tv_usec = a->tv_usec + b->tv_usec;
}

/**
 * Compute integer multiple of given struct timeval.
 *
 * \param mult The integer value to multiply with.
 * \param tv The timevalue to multiply.
 *
 * \param result Contains \a mult * \a tv on return.
 */
void tv_scale(const unsigned long mult, const struct timeval *tv,
		struct timeval *result)
{
	uint64_t x = ((uint64_t)tv->tv_sec * 1000 * 1000 + tv->tv_usec) * mult;

	result->tv_sec = x / 1000 / 1000;
	result->tv_usec = x % (1000 * 1000);
}

/**
 * Compute a fraction of given struct timeval.
 *
 * \param divisor The integer value to divide by.
 * \param tv The timevalue to divide.
 * \param result Contains (1 / mult) * tv on return.
 */
void tv_divide(const unsigned long divisor, const struct timeval *tv,
		struct timeval *result)
{
	uint64_t x = ((uint64_t)tv->tv_sec * 1000 * 1000 + tv->tv_usec) / divisor;

	result->tv_sec = x / 1000 / 1000;
	result->tv_usec = x % (1000 * 1000);
}

/**
 * Compute a convex combination of two time values.
 *
 * \param a The first coefficient.
 * \param tv1 The first time value.
 * \param b The second coefficient.
 * \param tv2 The second time value.
 * \param result Contains the convex combination upon return.
 *
 * Compute x := (a * tv1 + b * tv2) / (|a| + |b|) and store |x| in \a result.
 * Both \a a and \a b may be negative.
 *
 * \return Zero, 1 or -1, if \a x is zero, positive or negative, respectively.
 */
int tv_convex_combination(const long a, const struct timeval *tv1,
		const long b, const struct timeval *tv2,
		struct timeval *result)
{
	struct timeval tmp1, tmp2, tmp3;
	int ret = 1;
	unsigned long a1, b1;

	if (a == 0 && b == 0) {
		result->tv_sec = 0;
		result->tv_usec = 0;
		return 0;
	}
	a1 = PARA_ABS(a);
	b1 = PARA_ABS(b);
	tv_scale(a1, tv1, &tmp1);
	tv_scale(b1, tv2, &tmp2);
	if ((a > 0 && b < 0) || (a < 0 && b > 0)) /* subtract */
		ret = tv_diff(&tmp1, &tmp2, &tmp3);
	else
		tv_add(&tmp1, &tmp2, &tmp3);
	tv_divide(a1 + b1, &tmp3, result);
	if (!a || !b) {
		if (a + b < 0)
			ret = -1;
	} else if (a < 0)
		ret = -ret;
	return ret;
}

/**
 * Compute when to send a chunk of an audio file.
 *
 * \param chunk_num The number of the chunk.
 * \param chunk_tv The duration of one chunk.
 * \param stream_start When the first chunk was sent.
 * \param result The time when to send chunk number \a chunk_num.
 *
 * This function computes \a stream_start + \a chunk_num * \a chunk_time.
 */
void compute_chunk_time(long unsigned chunk_num,
		struct timeval *chunk_tv, struct timeval *stream_start,
		struct timeval *result)
{
	struct timeval tmp;

	tv_scale(chunk_num, chunk_tv, &tmp);
	tv_add(&tmp, stream_start, result);
}

/**
 * Retrieve the time of the realtime clock.
 *
 * \param tv Where to store the result.
 *
 * Gets the current value of the system-wide real-time clock (identified by id
 * \p CLOCK_REALTIME). If \a tv is \p NULL, the value is stored in a static
 * buffer, otherwise it is stored at the location given by \a tv.
 *
 * \return This function aborts on errors. On success it returns a pointer to
 * memory containing the current time.
 *
 * \sa clock_gettime(2), gettimeofday(2).
 */
struct timeval *clock_get_realtime(struct timeval *tv)
{
	static struct timeval user_friendly;

	if (!tv)
		tv = &user_friendly;
#ifdef HAVE_CLOCK_GETTIME
	{
		struct timespec t;
		int ret;

		ret = clock_gettime(CLOCK_REALTIME, &t);
		assert(ret == 0);
		tv->tv_sec = t.tv_sec;
		tv->tv_usec = t.tv_nsec / 1000;
	}
#else
	#include <sys/time.h>
	gettimeofday(tv, NULL);
#endif /* HAVE_CLOCK_GETTIME */
	return tv;
}
